unit Unit1;

interface

uses
  SysUtils, Classes, Controls, Forms, Midi, StdCtrls, SyncObjs, CheckLst, RM_1800,
  ComCtrls;

type
  TForm1 = class(TForm)
    Memo_Log: TMemo;
    lbxInputDevices: TCheckListBox;
    StatusBar1: TStatusBar;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure lbxInputDevicesClickCheck(Sender: TObject);
  private
    fCriticalSection: TCriticalSection;
  public
    procedure DoMidiInData( const aDeviceIndex: integer; const aStatus, aData1, aData2: byte );
    procedure DoSysExData( const aDeviceIndex: integer; const aStream: TMemoryStream );
  end;

var  Form1: TForm1;

implementation
{$R *.dfm}

///////////////////////////////////////////////////////////
procedure TForm1.DoMidiInData(const aDeviceIndex: integer; const aStatus,
  aData1, aData2: byte);
begin
  // skip active sensing signals from keyboard
  if aStatus = $FE then Exit;

  fCriticalSection.Acquire;
  Memo_Log.Lines.BeginUpdate;
  try

    // print the message log
    Memo_Log.Lines.Add (Format( '[%s] %s: <Status> %.2x, <Data 1> %.2x <Data 2> %.2x',
      [ FormatDateTime( 'HH:NN:SS.ZZZ', now ),
        MidiInput.Devices[aDeviceIndex],
        aStatus,
        aData1,
        aData2 ] ));
  finally
    Memo_Log.Lines.EndUpdate;
    fCriticalSection.Leave;
  end;
end;
///////////////////////////////////////////////////////////

procedure TForm1.FormCreate(Sender: TObject);
begin
  fCriticalSection := TCriticalSection.Create;
  lbxInputDevices.Items.Assign( MidiInput.Devices );

  MidiInput.OnMidiData := DoMidiInData;
  MidiInput.OnSysExData := DoSysExData;

  case lbxInputDevices.Items.Count of
       0: begin
           Memo_Log.Lines.Add('MIDI IN   .   .');
          end;
       1: begin
            lbxInputDevices.ItemIndex:=0;
            lbxInputDevices.Checked[0]:=True;
            lbxInputDevicesClickCheck (nil);
          end;
   end;
end;
///////////////////////////////////////////////////////////

procedure TForm1.FormDestroy(Sender: TObject);
begin
  FreeAndNil( fCriticalSection );
end;
///////////////////////////////////////////////////////////

procedure TForm1.lbxInputDevicesClickCheck(Sender: TObject);
begin
  if lbxINputDevices.Checked[ lbxInputDevices.ItemIndex ] then
    MidiInput.Open( lbxInputDevices.ItemIndex )
  else
    MidiInput.Close( lbxInputDevices.ItemIndex )
end;
///////////////////////////////////////////////////////////

procedure TForm1.DoSysExData(const aDeviceIndex: integer;
  const aStream: TMemoryStream);
  var SysExStr:string; KeyCode:LongWord;
begin
  fCriticalSection.Acquire;
  Memo_Log.Lines.BeginUpdate;
  try
    KeyCode:= SysEx_GetKeyCode( aStream ); //    . 0 = 
    SysExStr:=SysExStreamToStr( aStream ); //    

    Winamp_Control (KeyCode);

    Memo_Log.Lines.Add ( Format( '[%s] %s: <Bytes> %d <SysEx> %s = %.8x = %s',
      [ FormatDateTime( 'HH:NN:SS.ZZZ', now ),
        MidiInput.Devices[aDeviceIndex],
        aStream.Size,
        SysExStr,
        KeyCode,
        KeyCodeToStr (KeyCode) ] ));
        
  finally
    Memo_Log.Lines.EndUpdate;
    fCriticalSection.Leave;
  end;
end;
///////////////////////////////////////////////////////////

end.
